#' Draw barplot of cluster abundance within stimulated clusters
#' Project: AZIMUT
#' Author: Nicolas Vallet

#' Require
library(Seurat)
library(tidyverse)
library(reshape2)

#' set paths 
path_to_rds  = "~/scrna_files"
path_to_save = "~/tmp"

#' Read RDS file with clustered
setwd(path_to_rds)
prolif = readRDS( "prolif_adtclust_cyclecell.RData" )

#' Subset stim clusters
stim_clusters = c(0,1,9,15,5,19,3,7,17,8,12,25)
Idents(prolif) = prolif@meta.data$seurat_clusters
stim = subset(prolif, idents = stim_clusters)
Idents(stim) = stim@meta.data$condition
stim_azm = subset(stim, idents = "AZM")
stim_ctrl = subset(stim, idents = "CTRL")

#' Get the prop table of cell in clusters among each condition
## get in AZM
prop_azm = prop.table( table(stim_azm@meta.data$sample_id, stim_azm@meta.data$seurat_clusters),1 )*100 
prop_azm = as.data.frame.matrix(prop_azm) %>%
    select_if(colSums(.) != 0 ) %>%
    add_column(group = "AZM") %>%
    rownames_to_column("sample_id")

## get in CTRL
prop_ctrl = prop.table( table(stim_ctrl@meta.data$sample_id, stim_ctrl@meta.data$seurat_clusters),1 )*100 
prop_ctrl = as.data.frame.matrix(prop_ctrl) %>%
    select_if(colSums(.) != 0 ) %>%
    add_column(group = "CTRL") %>%
    rownames_to_column("sample_id")

## merge
prop_tot = rbind(prop_azm, prop_ctrl) %>%
    melt(id.vars = c("group","sample_id")) %>%
    mutate(variable = paste0("Cluster",variable)) %>%
    rename(cluster = variable) %>%
    mutate(cluster = fct_relevel(cluster, # order as heatmaps
                                 "Cluster25", "Cluster12", "Cluster8", "Cluster17", "Cluster7", "Cluster19", "Cluster3", "Cluster5", "Cluster15", "Cluster9", "Cluster1", "Cluster0"
                                 ))  


#' Draw the plot
plot = prop_tot %>%
    mutate(cluster = fct_rev(cluster)) %>%
    ggplot(aes(x = value, y = group, fill = group)) + 
    facet_grid(cluster ~.) +
    geom_boxplot(outlier.shape = NA, coef = 0, color = "black", ) +
    scale_x_sqrt(breaks=seq(0,65,5)) +
    theme_classic() +
    theme(axis.title = element_blank(),
          axis.text  = element_text(face = "bold", color = "black"),
          axis.text.x= element_text(angle = 45),
          axis.ticks = element_line(linetype = "solid", color = "black"),
          legend.position = "none") +
    scale_fill_manual(values = c("#6a3d9aff", "#6a3d9993"))
   


#' Export to pdf
setwd( path_to_save )
pdf("boxplot_azmpla.pdf", width = 2 , height = 4 )
print(plot)
dev.off()
